import React, { useEffect, useContext } from "react";
import useUrlState from "@ahooksjs/use-url-state";
import { Box, IconButton, Typography, Stack } from "@mui/material";
import ArrowBackIcon from "@mui/icons-material/ArrowBack";
import { HostBadge } from "../../../components/host/HostBadge";
import {
  KeyValueTable,
  SimpleAccordion,
  GlobalAppBarContext,
  AlertBannerList,
  ExternalLink
} from "@ess-ics/ce-ui-common";
import { LokiPanel } from "../../../components/common/Loki/LokiPanel";
import { useNavigate } from "react-router-dom";
import { applicationTitle } from "../../../components/common/Helper";
import AccessControl from "../../../components/auth/AccessControl";
import {
  serialize,
  deserialize
} from "../../../components/common/URLState/URLState";
import { HostDetailsTable } from "./HostDetailsTable";
import { HostJobsSection } from "./HostJobsSection";
import { HostIocSection } from "./HostIocSection";

export function HostDetailsView({ hostId, host }) {
  const { setTitle } = useContext(GlobalAppBarContext);
  const navigate = useNavigate();
  const [urlState, setUrlState] = useUrlState();

  useEffect(() => {
    if (host && host.netBoxHost) {
      setTitle(applicationTitle("Host Details: " + host.netBoxHost.name));
    }
  }, [host, setTitle]);

  useEffect(() => {
    if (Object.keys(urlState).length === 0) {
      setUrlState(
        {
          iocs_rows: 20,
          iocs_page: 0,
          details_open: false,
          job_log_open: false,
          job_log_rows: 20,
          job_log_page: 0
        },
        { navigateMode: "replace" }
      );
    }
  }, [setUrlState, urlState]);

  const handleClick = (event) => {
    navigate(-1);
  };

  return (
    <Stack gap={2}>
      <Box>
        <IconButton
          color="inherit"
          onClick={handleClick}
          size="large"
        >
          <ArrowBackIcon />
        </IconButton>
      </Box>
      {host ? (
        <>
          <AlertBannerList alerts={host?.alerts ?? []} />
          <HostBadge host={host} />
        </>
      ) : null}
      <Stack>
        <HostIocSection
          hostId={hostId}
          rows={deserialize(urlState.iocs_rows)}
          page={deserialize(urlState.iocs_page)}
          onUrlStateChange={(params) => setUrlState(params)}
        />
      </Stack>
      {hostId ? (
        <HostJobsSection
          hostId={hostId}
          rows={deserialize(urlState.job_log_rows)}
          page={deserialize(urlState.job_log_page)}
          expanded={deserialize(urlState.job_log_open)}
          onUrlStateChange={(params) => setUrlState(params)}
        />
      ) : null}

      <AccessControl
        allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
        renderNoAccess={() => <></>}
      >
        {host ? (
          <SimpleAccordion
            summary="Host details"
            expanded={deserialize(urlState.details_open)}
            onChange={(event, expanded) =>
              setUrlState({ details_open: serialize(expanded) })
            }
            sx={{ marginTop: "0 !important" }}
          >
            <HostDetailsTable host={host} />
          </SimpleAccordion>
        ) : null}
      </AccessControl>

      <AccessControl
        allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
        renderNoAccess={() => <></>}
      >
        <Stack gap={2}>
          <Typography variant="h3">Host log stream</Typography>
          {host ? (
            <LokiPanel
              host={host}
              isSyslog
              isDeployed
            />
          ) : null}
        </Stack>
      </AccessControl>
      <KeyValueTable
        obj={{
          "Host Configuration": (
            <ExternalLink
              href={
                host?.netBoxHost.vm
                  ? `${window.NETBOX_ADDRESS}/virtualization/virtual-machines/${host?.netBoxHost.id}`
                  : `${window.NETBOX_ADDRESS}/dcim/devices/${host?.netBoxHost.id}`
              }
              aria-label="Host Configuration"
            >
              {" "}
              {host?.netBoxHost.vm
                ? `${window.NETBOX_ADDRESS}/virtualization/virtual-machines/${host?.netBoxHost.id}`
                : `${window.NETBOX_ADDRESS}/dcim/devices/${host?.netBoxHost.id}`}
            </ExternalLink>
          ),
          "Host Metrics": (
            <ExternalLink
              href={`https://grafana.tn.esss.lu.se/d/5zJT23xWz/node-exporter-full?orgId=1&var-node=${host?.netBoxHost.fqdn}`}
              aria-label="Host Metrics"
            >
              {`https://grafana.tn.esss.lu.se/d/5zJT23xWz/node-exporter-full?orgId=1&var-node=${host?.netBoxHost.fqdn}`}
            </ExternalLink>
          )
        }}
        variant="overline"
      />
    </Stack>
  );
}
